/*
 * Simulator of microcontrollers (bit_inst.cc)
 *
 * Copyright (C) 1999,99 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of microcontroller simulator: ucsim.

UCSIM is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UCSIM is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UCSIM; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

// local
#include "avrcl.h"
#include "regsavr.h"


/*
 * Instruction: SEC
 *----------------------------------------------------------------------------
 */

int
cl_avr::sec(void)
{
  write_mem(MEM_SFR, SREG, BIT_C | read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: CLC
 *----------------------------------------------------------------------------
 */

int
cl_avr::clc(void)
{
  write_mem(MEM_SFR, SREG, ~BIT_C & read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: SEN
 *----------------------------------------------------------------------------
 */

int
cl_avr::sen(void)
{
  write_mem(MEM_SFR, SREG, BIT_N | read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: CLN
 *----------------------------------------------------------------------------
 */

int
cl_avr::cln(void)
{
  write_mem(MEM_SFR, SREG, ~BIT_N & read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: SEZ
 *----------------------------------------------------------------------------
 */

int
cl_avr::sez(void)
{
  write_mem(MEM_SFR, SREG, BIT_Z | read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: CLI
 *----------------------------------------------------------------------------
 */

int
cl_avr::clz(void)
{
  write_mem(MEM_SFR, SREG, ~BIT_Z & read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: SEI
 *----------------------------------------------------------------------------
 */

int
cl_avr::sei(void)
{
  write_mem(MEM_SFR, SREG, BIT_I | read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: CLI
 *----------------------------------------------------------------------------
 */

int
cl_avr::cli(void)
{
  write_mem(MEM_SFR, SREG, ~BIT_I & read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: SES
 *----------------------------------------------------------------------------
 */

int
cl_avr::ses(void)
{
  write_mem(MEM_SFR, SREG, BIT_S | read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: CLS
 *----------------------------------------------------------------------------
 */

int
cl_avr::cls(void)
{
  write_mem(MEM_SFR, SREG, ~BIT_S & read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: SEV
 *----------------------------------------------------------------------------
 */

int
cl_avr::sev(void)
{
  write_mem(MEM_SFR, SREG, BIT_V | read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: CLV
 *----------------------------------------------------------------------------
 */

int
cl_avr::clv(void)
{
  write_mem(MEM_SFR, SREG, ~BIT_V & read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: SET
 *----------------------------------------------------------------------------
 */

int
cl_avr::set(void)
{
  write_mem(MEM_SFR, SREG, BIT_T | read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: CLT
 *----------------------------------------------------------------------------
 */

int
cl_avr::clt(void)
{
  write_mem(MEM_SFR, SREG, ~BIT_T & read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: SEH
 *----------------------------------------------------------------------------
 */

int
cl_avr::seh(void)
{
  write_mem(MEM_SFR, SREG, BIT_H | read_mem(MEM_SFR, SREG));
  return(resGO);
}


/*
 * Instruction: CLH
 *----------------------------------------------------------------------------
 */

int
cl_avr::clh(void)
{
  write_mem(MEM_SFR, SREG, ~BIT_H & read_mem(MEM_SFR, SREG));
  return(resGO);
}


int
cl_avr::cbi_A_b(void)
{
  return(resGO);
}


int
cl_avr::sbic_A_b(void)
{
  return(resGO);
}


int
cl_avr::sbi_A_b(void)
{
  return(resGO);
}


int
cl_avr::sbis_A_b(void)
{
  return(resGO);
}


/* End of bit_inst.cc */
