/*
 * Simulator of microcontrollers (arith.cc)
 *
 * Copyright (C) 1999,99 Drotos Daniel, Talker Bt.
 * 
 * To contact author send email to drdani@mazsola.iit.uni-miskolc.hu
 *
 */

/* This file is part of microcontroller simulator: ucsim.

UCSIM is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

UCSIM is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with UCSIM; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA. */
/*@1@*/

#include "ddconfig.h"

#include <stdio.h>

// local
#include "uc51cl.h"
#include "regs51.h"


/*
 * 0x03 1 12 RR A
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_rr(uchar code)
{
  uchar acc;

  acc= read_mem(MEM_SFR, event_at.ws= event_at.rs= ACC); 
  if (acc & 0x01)
    write_mem(MEM_SFR, ACC, (acc >> 1) & 0x80);
  else
    write_mem(MEM_SFR, ACC, acc >> 1);
  return(resGO);
}


/*
 * 0x13 1 12 RRC A
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_rrc(uchar code)
{
  bool cy;
  uchar acc;

  cy= GET_C;
  SET_C((acc= read_mem(MEM_SFR, ACC)) & 0x01);
  event_at.ws= event_at.rs= ACC;
  acc>>= 1;
  if (cy)
    acc|= 0x80;
  write_mem(MEM_SFR, ACC, acc);
  return(resGO);
}


/*
 * 0x23 1 12 RL A
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_rl(uchar code)
{
  uchar acc;

  acc= read_mem(MEM_SFR, event_at.ws= event_at.rs= ACC);
  if (acc & 0x80)
    write_mem(MEM_SFR, ACC, (acc << 1 ) | 0x01);
  else
    write_mem(MEM_SFR, ACC, acc << 1);
  return(resGO);
}


/*
 * 0x24 2 12 ADD A,#data
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_add_a_$data(uchar code)
{
  uchar data, acc;
  bool newC, newA, c6;

  data= fetch();
  acc = get_mem(MEM_SFR, ACC);
  newC= (((uint)acc+(uint)(data)) > 255)?0x80:0;
  newA= ((acc&0x0f)+(data&0x0f)) & 0xf0;
  c6  = ((acc&0x7f)+(data&0x7f)) & 0x80;
  event_at.ws= ACC;
  write_mem(MEM_SFR, ACC, acc+data);
  SET_C(newC);
  SET_BIT(newC ^ c6, PSW, bmOV);
  SET_BIT(newA, PSW, bmAC);
  return(resGO);
}


/*
 * 0x25 2 12 ADD A,addr
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_add_a_addr(uchar code)
{
  uchar data, acc;
  bool newC, newA, c6;

  data= read(get_direct(fetch(), &event_at.ri, &event_at.rs));
  acc = get_mem(MEM_SFR, ACC);
  newC= (((uint)acc+(uint)(data)) > 255)?0x80:0;
  newA= ((acc&0x0f)+(data&0x0f)) & 0xf0;
  c6  = ((acc&0x7f)+(data&0x7f)) & 0x80;
  event_at.ws= ACC;
  write_mem(MEM_SFR, ACC, acc+data);
  SET_C(newC);
  SET_BIT(newC ^ c6, PSW, bmOV);
  SET_BIT(newA, PSW, bmAC);
  return(resGO);
}


/*
 * 0x26-0x27 1 12 ADD A,@Ri
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_add_a_$ri(uchar code)
{
  uchar data, *addr, acc;
  bool newC, newA, c6;
  int res;

  addr= get_indirect(event_at.ri= *(get_reg(code & 0x01)), &res);
  acc = get_mem(MEM_SFR, ACC);
  data= *addr;
  newC= (((uint)acc+(uint)data) > 255)?0x80:0;
  newA= ((acc&0x0f)+(data&0x0f)) & 0xf0;
  c6  = ((acc&0x7f)+(data&0x7f)) & 0x80;
  event_at.ws= ACC;
  write_mem(MEM_SFR, ACC, acc+data);
  SET_C(newC);
  SET_BIT(newC ^ c6, PSW, bmOV);
  SET_BIT(newA, PSW, bmAC);
  return(res);
}


/*
 * 0x28-0x2f 1 12 ADD A,Rn
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_add_a_rn(uchar code)
{
  uchar data, acc;
  bool newC, newA, c6;

  data= *(get_reg(code & 0x07, &event_at.ri));
  acc = get_mem(MEM_SFR, ACC);
  newC= (((uint)acc+(uint)data) > 255)?0x80:0;
  newA= ((acc&0x0f)+(data&0x0f)) & 0xf0;
  c6  = ((acc&0x7f)+(data&0x7f)) & 0x80;
  event_at.ws= ACC;
  write_mem(MEM_SFR, ACC, acc+data);
  SET_C(newC);
  SET_BIT(newC ^ c6, PSW, bmOV);
  SET_BIT(newA, PSW, bmAC);
  return(resGO);
}


/*
 * 0x33 1 12 RLC A
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_rlc(uchar code)
{
  bool cy;
  uchar acc;

  cy= GET_C;
  SET_C((acc= read_mem(MEM_SFR, event_at.rs= ACC)) & 0x80);
  acc<<= 1;
  if (cy)
    acc|= 0x01;
  write_mem(MEM_SFR, event_at.ws= ACC, acc);
  return(resGO);
}


/*
 * 0x34 2 12 ADDC A,#data
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_addc_a_$data(uchar code)
{
  uchar data, acc;
  bool orgC, newC, newA, c6;

  data= fetch();
  acc = get_mem(MEM_SFR, ACC);
  newC= (((uint)acc+(uint)data+((orgC= GET_C)?1:0)) > 255)?0x80:0;
  newA= ((acc&0x0f)+(data&0x0f)+(orgC?1:0)) & 0xf0;
  c6  = ((acc&0x7f)+(data&0x7f)+(orgC?1:0)) & 0x80;
  write_mem(MEM_SFR, event_at.ws= ACC, acc + data + (orgC?1:0));
  SET_C(newC);
  SET_BIT(newC ^ c6, PSW, bmOV);
  SET_BIT(newA, PSW, bmAC);
  return(resGO);
}


/*
 * 0x35 2 12 ADDC A,addr
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_addc_a_addr(uchar code)
{
  uchar data, acc;
  bool orgC, newC, newA, c6;

  data= read(get_direct(fetch(), &event_at.ri, &event_at.rs));
  acc = get_mem(MEM_SFR, ACC);
  newC= (((uint)acc+(uint)data+((orgC= GET_C)?1:0)) > 255)?0x80:0;
  newA= ((acc&0x0f)+(data&0x0f)+(orgC?1:0)) & 0xf0;
  c6  = ((acc&0x7f)+(data&0x7f)+(orgC?1:0)) & 0x80;
  write_mem(MEM_SFR, event_at.ws= ACC, acc + data + (orgC?1:0));
  SET_C(newC);
  SET_BIT(newC ^ c6, PSW, bmOV);
  SET_BIT(newA, PSW, bmAC);
  return(resGO);
}


/*
 * 0x36-0x37 1 12 ADDC A,@Ri
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_addc_a_$ri(uchar code)
{
  uchar data, *addr, acc;
  bool orgC, newC, newA, c6;
  int res;

  addr= get_indirect(event_at.ri= *(get_reg(code & 0x01)), &res);
  acc = get_mem(MEM_SFR, ACC);
  data= *addr;
  newC= (((uint)acc+(uint)data+((orgC= GET_C)?1:0)) > 255)?0x80:0;
  newA= ((acc&0x0f)+(data&0x0f)+(orgC?1:0)) & 0xf0;
  c6  = ((acc&0x7f)+(data&0x7f)+(orgC?1:0)) & 0x80;
  write_mem(MEM_SFR, event_at.ws= ACC, acc + data + (orgC?1:0));
  SET_C(newC);
  SET_BIT(newC ^ c6, PSW, bmOV);
  SET_BIT(newA, PSW, bmAC);
  return(res);
}


/*
 * 0x38-0x3f 1 12 ADDC A,Rn
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_addc_a_rn(uchar code)
{
  uchar data, acc;
  bool orgC, newC, newA, c6;

  data= *(get_reg(code & 0x07, &event_at.ri));
  acc = get_mem(MEM_SFR, ACC);
  newC= (((uint)acc+(uint)data+((orgC= GET_C)?1:0)) > 255)?0x80:0;
  newA= ((acc&0x0f)+(data&0x0f)+(orgC?1:0)) & 0xf0;
  c6  = ((acc&0x7f)+(data&0x7f)+(orgC?1:0)) & 0x80;
  write_mem(MEM_SFR, event_at.ws= ACC, acc + data + (orgC?1:0));
  SET_C(newC);
  SET_BIT(newC ^ c6, PSW, bmOV);
  SET_BIT(newA, PSW, bmAC);
  return(resGO);
}


/*
 * 0x84 1 48 DIV AB
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_div_ab(uchar code)
{
  uchar temp, psw, b, acc;

  psw= get_mem(MEM_SFR, PSW);
  psw&= ~bmCY;
  if (!(b= get_mem(MEM_SFR, event_at.rs= B)))
    psw|= bmOV;
  else
    {
      psw&= ~bmOV;
      temp= (acc= get_mem(MEM_SFR, ACC)) / b;
      write_mem(MEM_SFR, B, acc % b);
      write_mem(MEM_SFR, event_at.ws= ACC, temp);
    }
  write_mem(MEM_SFR, PSW, psw);
  tick(3);
  return(resGO);
}


/*
 * 0x94 2 12 SUBB A,#data
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_subb_a_$data(uchar code)
{
  uchar data, d, acc;
  bool newC, newA, c6;

  data= fetch();
  acc = get_mem(MEM_SFR, ACC);
  d= ~data + (GET_C?0:1);
  newC= (acc < data+(GET_C?1:0))?0x80:0;
  newA= !(((acc&0x0f)+(d&0x0f)) & 0xf0);
  c6  = (((acc&0x7f)+(d&0x7f)) & 0x80)?0:0x80;
  acc-= data;
  if (GET_C)
    acc--;
  write_mem(MEM_SFR, event_at.ws= ACC, acc);
  SET_C(newC);
  SET_BIT(newC ^ c6, PSW, bmOV);
  SET_BIT(newA, PSW, bmAC);
  return(resGO);
}


/*
 * 0x95 2 12 SUBB A,addr
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_subb_a_addr(uchar code)
{
  uchar *addr, data, d, acc;
  bool newC, newA, c6;

  addr= get_direct(fetch(), &event_at.ri, &event_at.rs);
  acc = get_mem(MEM_SFR, ACC);
  data= read(addr);
  d= ~data + (GET_C?0:1);
  newC= (acc < data+(GET_C?1:0))?0x80:0;
  newA= !(((acc&0x0f)+(d&0x0f)) & 0xf0);
  c6  = (((acc&0x7f)+(d&0x7f)) & 0x80)?0:0x80;
  acc-= data;
  event_at.ws= ACC;
  if (GET_C)
    acc--;
  write_mem(MEM_SFR, ACC, acc);
  SET_C(newC);
  SET_BIT(newC ^ c6, PSW, bmOV);
  SET_BIT(newA, PSW, bmAC);
  return(resGO);
}


/*
 * 0x96-0x97 1 12 SUBB A,@Ri
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_subb_a_$ri(uchar code)
{
  uchar data, d, acc;
  bool newC, newA, c6;
  int res;

  data= *(get_indirect(event_at.ri= *(get_reg(code & 0x01)), &res));
  acc = get_mem(MEM_SFR, ACC);
  d= ~data + (GET_C?0:1);
  newC= (acc < data+(GET_C?1:0))?0x80:0;
  newA= !(((acc&0x0f)+(d&0x0f)) & 0xf0);
  c6  = (((acc&0x7f)+(d&0x7f)) & 0x80)?0:0x80;
  acc-= data;
  event_at.ws= ACC;
  if (GET_C)
    acc--;
  write_mem(MEM_SFR, ACC, acc);
  SET_C(newC);
  SET_BIT(newC ^ (acc & 0x80), PSW, bmOV);
  SET_BIT(newA, PSW, bmAC);
  return(res);
}


/*
 * 0x98-0x9f 1 12 SUBB A,Rn
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_subb_a_rn(uchar code)
{
  uchar data, d, acc;
  bool newC, newA, c6;

  data= *(get_reg(code & 0x07, &event_at.ri));
  acc = get_mem(MEM_SFR, ACC);
  d= ~data + (GET_C?0:1);
  newC= (acc < data+(GET_C?1:0))?0x80:0;
  newA= !(((acc&0x0f)+(d&0x0f)) & 0xf0);
  c6  = (((acc&0x7f)+(d&0x7f)) & 0x80)?0:0x80;
  acc-= data;
  event_at.ws= ACC;
  if (GET_C)
    acc--;
  write_mem(MEM_SFR, ACC, acc);
  SET_C(newC);
  SET_BIT(newC ^ (acc & 0x80), PSW, bmOV);
  SET_BIT(newA, PSW, bmAC);
  return(resGO);
}


/*
 * 0xa4 1 48 MUL AB
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_mul_ab(uchar code)
{
  uint temp, psw, acc, b;

  psw= get_mem(MEM_SFR, PSW);
  psw&= ~bmCY;
  temp= (acc= read_mem(MEM_SFR, ACC)) * (b= read_mem(MEM_SFR, B));
  write_mem(MEM_SFR, event_at.ws= ACC, temp & 0xff);
  write_mem(MEM_SFR, event_at.rs= B, (temp >> 8) & 0xff);
  SET_BIT(get_mem(MEM_SFR, B), PSW, bmOV);
  tick(3);
  return(resGO);
}


/*
 * 0xd4 1 12 DA A
 *____________________________________________________________________________
 *
 */

int
t_uc51::inst_da_a(uchar code)
{
  uchar acc, psw;

  acc= get_mem(MEM_SFR, ACC);
  psw= get_mem(MEM_SFR, PSW);
  event_at.ws= ACC;
  if ((acc & 0x0f) > 9 ||
      (psw & bmAC))
    {
      if (((uint)acc+(uint)0x06) > 255)
	psw|= bmCY;
      acc+= 0x06;
    }
  if ((acc & 0xf0) > 0x90 ||
      (psw & bmCY))
    {
      if (((uint)acc+(uint)0x60) > 255)
	psw|= bmCY;
      acc+= 0x60;
    }
  write_mem(MEM_SFR, ACC, acc);
  write_mem(MEM_SFR, PSW, psw);
  return(resGO);
}


/* End of arith.cc */
